diff a/src/hotspot/share/oops/klass.inline.hpp b/src/hotspot/share/oops/klass.inline.hpp
--- a/src/hotspot/share/oops/klass.inline.hpp
+++ b/src/hotspot/share/oops/klass.inline.hpp
@@ -50,10 +50,28 @@
 // of this function must ensure that a safepoint doesn't happen while interpreting the return value.
 inline bool Klass::is_loader_alive() const {
   return class_loader_data()->is_alive();
 }
 
+inline void Klass::set_prototype_header(markWord header) {
+  assert(!is_inline_klass() || header.is_inline_type(), "Unexpected prototype");
+  assert(_prototype_header.value() == 0 || _prototype_header == markWord::prototype(),
+         "Prototype already set");
+#ifdef _LP64
+    assert(header == markWord::prototype() ||
+           header.is_inline_type() ||
+           header.is_flat_array() ||
+           header.is_null_free_array(),
+           "unknown prototype header");
+#else
+    assert(header == markWord::prototype() ||
+           header.is_inline_type(),
+           "unknown prototype header");
+#endif
+  _prototype_header = header;
+}
+
 inline oop Klass::java_mirror() const {
   return _java_mirror.resolve();
 }
 
 inline oop Klass::java_mirror_no_keepalive() const {
